Attribute VB_Name = "BezierStuff"
Option Explicit

Public Type PointF
X As Single
Y As Single
End Type

' Draw a closed cardinal spline built from connected Bezier curves.
Public Sub DrawClosedCurve(ByVal pic As Object, ByVal dt As Single, ByVal tension As Single, pts() As PointF)
Dim control_scale As Single
Dim pt As PointF
Dim pt_before As PointF
Dim pt_after As PointF
Dim pt_after2 As PointF
Dim Di As PointF
Dim DiPlus1 As PointF
Dim p1 As PointF
Dim p2 As PointF
Dim p3 As PointF
Dim p4 As PointF
Dim i As Integer
Dim num_pts As Integer

    control_scale = CSng(tension / 0.5 * 0.175)
    num_pts = UBound(pts) + 1
    For i = 0 To UBound(pts)
        pt_before = pts((i - 1 + num_pts) Mod num_pts)
        pt = pts(i)
        pt_after = pts((i + 1) Mod num_pts)
        pt_after2 = pts((i + 2) Mod num_pts)

        p1 = pt
        p4 = pt_after

        Di.X = pt_after.X - pt_before.X
        Di.Y = pt_after.Y - pt_before.Y
        p2.X = pt.X + control_scale * Di.X
        p2.Y = pt.Y + control_scale * Di.Y

        DiPlus1.X = pt_after2.X - pts(i).X
        DiPlus1.Y = pt_after2.Y - pts(i).Y
        p3.X = pt_after.X - control_scale * DiPlus1.X
        p3.Y = pt_after.Y - control_scale * DiPlus1.Y

        DrawBezier pic, dt, p1, p2, p3, p4
    Next i
End Sub
' Return the minimum of the parameters.
Private Function Min(ParamArray values() As Variant)
Dim i As Integer
Dim result As Single

    result = values(LBound(values))
    For i = LBound(values) + 1 To UBound(values)
        If result > values(i) Then result = values(i)
    Next i

    Min = result
End Function
' Return the maximum of the parameters.
Private Function Max(ParamArray values() As Variant)
Dim i As Integer
Dim result As Single

    result = values(LBound(values))
    For i = LBound(values) + 1 To UBound(values)
        If result < values(i) Then result = values(i)
    Next i

    Max = result
End Function

' Parametric X function for drawing a degree 3 Bezier curve.
Private Function X(ByVal t As Single, ByVal x0 As Single, ByVal x1 As Single, ByVal x2 As Single, ByVal x3 As Single) As Single
    X = CSng( _
        x0 * (1 - t) ^ 3 + _
        x1 * 3 * t * (1 - t) ^ 2 + _
        x2 * 3 * t ^ 2 * (1 - t) + _
        x3 * t ^ 3 _
        )
End Function
' Parametric Y function for drawing a degree 3 Bezier curve.
Private Function Y(ByVal t As Single, ByVal y0 As Single, ByVal y1 As Single, ByVal y2 As Single, ByVal y3 As Single) As Single
    Y = CSng( _
        y0 * (1 - t) ^ 3 + _
        y1 * 3 * t * (1 - t) ^ 2 + _
        y2 * 3 * t ^ 2 * (1 - t) + _
        y3 * t ^ 3 _
        )
End Function

' Draw the Bezier curve.
Public Sub DrawBezier(ByVal pic As Object, ByVal dt As Single, pt0 As PointF, pt1 As PointF, pt2 As PointF, pt3 As PointF)
    ' Debugging code.
    ' Draw the control lines.
    Dim p As PictureBox
    pic.DrawStyle = vbDot
    pic.Line (pt0.X, pt0.Y)-(pt1.X, pt1.Y)
    pic.Line (pt2.X, pt2.Y)-(pt3.X, pt3.Y)
    pic.DrawStyle = vbSolid

    ' Draw the curve.
    Dim t As Single
    Dim x0 As Single
    Dim y0 As Single
    Dim x1 As Single
    Dim y1 As Single

    t = 0#
    x1 = X(t, pt0.X, pt1.X, pt2.X, pt3.X)
    y1 = Y(t, pt0.Y, pt1.Y, pt2.Y, pt3.Y)
    t = t + dt
    Do While t < 1#
        x0 = x1
        y0 = y1
        x1 = X(t, pt0.X, pt1.X, pt2.X, pt3.X)
        y1 = Y(t, pt0.Y, pt1.Y, pt2.Y, pt3.Y)
        pic.Line (x0, y0)-(x1, y1)
        t = t + dt
    Loop

    ' Connect to the final point.
    t = 1#
    x0 = x1
    y0 = y1
    x1 = X(t, pt0.X, pt1.X, pt2.X, pt3.X)
    y1 = Y(t, pt0.Y, pt1.Y, pt2.Y, pt3.Y)
    pic.Line (x0, y0)-(x1, y1)
End Sub
